#!/usr/bin/ruby
# @file   ndk_config.rb
# @author K.S.
#
# $Date: 2004/04/19 07:31:24 $
# $Id: ndk_config.rb,v 1.2 2004/04/19 07:31:24 ko1 Exp $
#
# Create : K.S. 04/04/17 16:50:33
#

module Nadoka
  class NDK_Logger
    def initialize level, config
      @level = level
      @config= config
      @dlog = @config.debug_log
    end
    
    def dlog msg
      if @config.loglevel >= 3
        logging(@dlog, msg)
      end
    end
    
    def slog msg
      logging(make_logfilename(@config.system_log), msg)
    end
    
    def clog ch, msg
      logfile = (@config.channel_info[ch] && @config.channel_info[ch][:log]) ||
                @config.default_log
      logfile = make_logfilename(logfile, ch)
      
      logging(logfile, msg)
    end

    def log msg
      if @config.loglevel >= 2
        logging(make_logfilename(@config.system_log), msg)
      end
    end

    def logging(io, msg)
      msg = "#{Time.now.strftime('%y/%m/%d-%H:%M:%S')} #{msg}"
      if io.respond_to? :puts
        io.puts msg
      else
        open(File.expand_path(@config.log_dir) + '/' + io, 'a'){|f|
          f.puts msg
        }
      end
    end

    def make_logfilename(tmpl, ch='')
      str = Time.now.strftime(tmpl)
      str.gsub!(/\$S/, @config.setting_name)
      str.gsub!(/\$C/, ch)
      str.tr!('\\/:*?<>|', '_')
      str
    end
  end
  
  class NDK_Config
    DefaultConfig = {
      # system
      # 0: quiet, 1: ..., 2: ..., 3: debug
      :loglevel => 3,
      :setting_name => 'DefaultSetting',
      
      # client server
      :client_server_port => 16666,
      :client_server_pass => 'passwd',

      # 
      :server_list        => [['127.0.0.1', 6667]],
      :server_passwd      => nil,
      :default_channels   => [],

      #
      :user       => 'testbot',
      :nick       => 'test_bot',
      :hostname   => '0',
      :servername => '*',
      :realname   => 'test bot on nadoka',

      #
      :channel_info => {}, 
      
      # log
      :default_log=> '$S-$C-%y%m.log',
      :system_log => '$S-system_log',
      :debug_log  => $stdout,
      
      # dirs
      :plugins_dir => './plugins',
      :log_dir     => './log',
      
      # bots
      :bots        => []
    }

    DefaultConfig.each{|e, _|
      eval %Q{
        def #{e}
          @config['#{e}'.intern]
        end
      }
    }
    
    def initialize rcfile
      load_config(rcfile || './nadokarc')
    end
    attr_reader :config
    
    def load_config(rcfile)
      load(rcfile) if rcfile
      @config = DefaultConfig.dup
      if defined? $NDK_Config
        $NDK_Config.each{|k,v|
          @config[k] = v
        }
        if svrs = $NDK_Config[:servers]
          svl = []
          svrs.each{|sv, ports|
            ports.each{|port|
              svl << [sv, port]
            }
          }
          @config[:server_list] = svl
        end
        if chs = $NDK_Config[:channel_info]
          dchs = []
          chs.each{|ch, setting|
            if setting[:default] == true
              dchs << ch
            end
          }
          @config[:default_channels] = dchs
        end
        
      end
      @logger = NDK_Logger.new(0, self)
    end

    # debug message
    def dlog msg
      @logger.dlog msg
    end
    
    # system message
    def slog msg
      @logger.slog msg
    end

    # normal message
    def log msg
      @logger.log msg
    end
    
    # channel message
    def clog ch, msg
      @logger.clog ch, msg
    end
    
  end
end

if $0 == __FILE__
  require 'pp'
  pp Nadoka::NDK_Config.new
end

